<?php
/*
==========================================================
	This software package is intended for use with 
	ExpressionEngine.	ExpressionEngine is Copyright © 
	2002-2009 EllisLab, Inc. 
	http://ellislab.com/
==========================================================
	THIS IS COPYRIGHTED SOFTWARE, All RIGHTS RESERVED.
	Written by: Justin Crawford
	Copyright (c) 2009 Hop Studios
	http://www.hopstudios.com/software/
--------------------------------------------------------
	Please do not distribute this software without written
	consent from the author.
==========================================================
	Files:
	- pi.publish_that.php
----------------------------------------------------------
	Purpose: 
	- adds an publish_that link to rendered templates, if user has permissions to edit 
==========================================================
*/

$plugin_info = array(
  'pi_name' => 'Publish That',
  'pi_version' => '1.0.0',
  'pi_author' => 'Justin Crawford (Hop Studios)',
  'pi_author_url' => 'http://www.hopstudios.com/software',
  'pi_description' => 'Adds a "publish that" link to various content',
  'pi_usage' => Publish_that::usage()
);

class Publish_that
{

	var $icon_path = '';
	var $session_required = FALSE;
	var $debug = FALSE;
	
	// constructor
	function Publish_that() 
	{
		global $PREFS, $TMPL;
		$this->icon_path = $PREFS->ini('theme_folder_url') . 'publish_that_themes/images/';
		$this->debug = ($TMPL->fetch_param('debug') == 'true') ? TRUE : FALSE;
		$this->_debug("initializing");
	}

	// _get_link: create an href and wrap it around an image
	function _get_link($url = '', $alt = '', $icon = 'default') 
	{
		$this->icon_path .= (($icon == '') ? 'default' : $icon) . '_icon.gif';
		$this->_debug("creating a link with '$url'");
		return '<a href="' . $url . '" title="' . $alt . '" target="_blank"><img src="' . $this->icon_path . '" height="14" width="14" style="padding-left: 4px;" alt="' . $alt . '" align="top" border="0" /></a>';
	}

	// _session_ok: check whether a session ID is necessary, and get it if it exists
	function _session_ok()
	{
		global $PREFS, $SESS, $DB;

		// check if if we need an admin session id
		if ($PREFS->ini('admin_session_type') != 'c')
		{
			$this->session_required = TRUE;

			// if we haven't already got one, get one and cache it
			if (! isset($SESS->cache['publish_that']['session_id']))
			{
				$query = $DB->query("SELECT session_id FROM exp_sessions WHERE member_id = '" . $SESS->userdata['member_id'] . "' AND admin_sess = '1'");		
				if ($query->num_rows > 0)
				{
					$this->_debug("found an admin session id -- caching it");
					$SESS->cache['publish_that']['session_id'] = $query->row['session_id'];
				}
				else
				{
					// if admin sessions are required, but we didn't find an admin session
					$this->_debug("admin session check: FALSE");
					return FALSE;
				}
			}
		}
		else
		{
			$this->session_required = FALSE;
		}
		
		// either we found an admin session, or we don't need one
		$this->_debug("admin session check: TRUE");
		return TRUE;
	}

	function _debug($msg)
	{
		if ($this->debug === TRUE)
		{
			// do something with debug msgs; this is rudimentary, but works.
			echo "publish_that -- " . $msg . "<br />";
		}
	}

	// authz: if superadmin, $can_publish
	function _authz_superadmin()
	{
		global $SESS;
		if ($SESS->userdata['group_id'] == 1)
		{
			$this->_debug("superadmin check: TRUE");
			return TRUE;
		}
		$this->_debug("superadmin check: FALSE");
		return FALSE;
	}

	// entry: return a link or a url for an publish_that:entry tag
	function entry() 
	{
		global $TMPL, $PREFS, $SESS, $DB;
		$this->_debug("handling an entry");
		$can_publish = FALSE;

		// there are three required tag parameters, all numeric
		if (is_numeric($weblog_id = ($TMPL->fetch_param('weblog_id'))))
		{

			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{
				if ($this->_authz_superadmin() === TRUE)
				{
					$can_publish = TRUE;
				}
				else 
				{
					// check the cache for authz for this weblog
					if (! isset($SESS->cache['publish_that']['weblog:' . $weblog_id]))
					{
	
						// authz: check that this person's group can publish that weblog
						$query = $DB->query("SELECT wt.weblog_id, wt.author_id FROM exp_weblog_member_groups wg, exp_weblog_titles wt WHERE wt.entry_id = " . $entry_id . " AND wg.weblog_id = wt.weblog_id AND wg.group_id = " . $SESS->userdata['group_id']);
	
						// cache the result of this authz check
						if ($query->num_rows > 0)
						{
							// they passed us a weblog/author id, but the DB is more definitive
							$weblog_id = $query->row['weblog_id'];
							$author_id = $query->row['author_id'];

							$SESS->cache['publish_that']['weblog:' . $weblog_id] = TRUE;
						}
						else
						{
							$SESS->cache['publish_that']['weblog:' . $weblog_id] = FALSE;
						}
						$this->_debug("caching result of weblog group authz for weblog $weblog_id");
					}
					
					// if we've cached affirmative authz for this weblog...
					if ($SESS->cache['publish_that']['weblog:' . $weblog_id] === TRUE)
					{
						$this->_debug("weblog group check: TRUE");
	
						// they must have these perms to edit 
						$can_publish = ($SESS->userdata['can_access_cp'] == 'y') && ($SESS->userdata['can_access_publish'] == 'y');	
						$this->_debug("control panel permission check: $can_publish");
					}
				}
			}
			
			// if we got through all the above an still $can_publish, then let's do it!
			if ($can_publish)
			{
				// if admin sessions are required, get the session id
				$session_param = ($this->session_required == TRUE) ? '&amp;S=' . $SESS->cache['publish_that']['session_id'] : '';

				// generate the url
				$url = $PREFS->core_ini['cp_url'] . '?C=publish&amp;M=entry_form&amp;weblog_id=' . $weblog_id . $session_param;

				// if invoked with two-tag, handle contents
				if ($TMPL->tagdata != '')
				{ 
					return str_replace('{publish_that_url}', $url, $TMPL->tagdata);  
				}
				else 
				{
					$alt = "Publish a new item";
					return $this->_get_link($url, $alt, $TMPL->fetch_param('icon'));
				}
			}
		}
		return;
	}

	// template: return a url or a link for an publish_that:template tag
	function template()
	{
		global $TMPL, $PREFS, $SESS, $DB;
		$this->_debug("handling a template");
		$can_publish = FALSE;

		if (is_numeric($template_group = ($TMPL->fetch_param('template_group'))))
		{

			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{

				if ($this->_authz_superadmin())
				{
					$can_publish = TRUE;
				}
				else 
				{
					// check the cache for a template group authz
					if (! isset($SESS->cache['publish_that']['template_group:' . $template_group]))
					{
						// try to get authz on this template group
						$query = $DB->query("SELECT tt.group_id FROM exp_template_member_groups tg, exp_templates tt WHERE tt.template_id = " . $template_id . " AND tg.template_group_id = tt.group_id AND tg.group_id = " . $SESS->userdata['group_id']);

						if ($query->num_rows > 0)
						{
							// they passed us a template group id, but the DB is more definitive
							$template_group = $query->row['group_id'];

							$SESS->cache['publish_that']['template_group:' . $template_group] = TRUE;
						}
						else
						{
							$SESS->cache['publish_that']['template_group:' . $template_group] = FALSE;
						}
						$this->_debug("caching result of template group authz for template group $template_group");
					}

					// if template group authz succeeded
					if ($SESS->cache['publish_that']['template_group:' . $template_group] === TRUE)
					{
						$this->_debug("template group check: TRUE");

						// check other required template permissions
						$can_publish = ($SESS->userdata['can_access_cp'] == 'y')	&& ($SESS->userdata['can_admin_templates'] == 'y');	
						$this->_debug("control panel permission check: $can_publish");
					}
				}
			}
		
			if ($can_publish)
			{
				// if admin sessions are required, get the session id
				$session_param = ($this->session_required == TRUE) ? '&amp;S=' . $SESS->cache['publish_that']['session_id'] : '';

				$url = $PREFS->core_ini['cp_url'] . '?C=templates&amp;M=new_templ_form&amp;id=' . $template_group . '&amp;tgpref=' . $template_group . $session_param;
				if ($TMPL->tagdata != '')
				{ 
					return str_replace('{publish_that_url}', $url, $TMPL->tagdata);  
				}
				else 
				{	
					$alt = "Publish a new template";
					return $this->_get_link($url, $alt, $TMPL->fetch_param('icon'));
				}
			}
		}
		return;
	}


	// category: return a url or a link for an publish_that:category tag
	function category()
	{
		global $TMPL, $PREFS, $DB, $SESS;
		$this->_debug("handling a category");
		$can_publish = FALSE;

		if (is_numeric($category_group = ($TMPL->fetch_param('category_group'))))
		{
			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{

				if ($this->_authz_superadmin() === TRUE)
				{
					$can_publish = TRUE;
				}
				else 
				{
					// they must have these perms to edit 
					$can_publish = ($SESS->userdata['can_access_cp'] == 'y')	&& ($SESS->userdata['can_access_admin'] == 'y') && ($SESS->userdata['can_admin_weblogs'] == 'y');
					$this->_debug("control panel permission check: $can_publish");
					// Note: there is a perm 'can_edit_categories', but it doesn't apply in this instance.  It's for editing categories on a particular entry.
				}
			
				if ($can_publish)
				{
					// if admin sessions are required, get the session id
					$session_param = ($this->session_required == TRUE) ? '&amp;S=' . $SESS->cache['publish_that']['session_id'] : '';

					$url = $PREFS->core_ini['cp_url'] . '?C=admin&amp;M=blog_admin&amp;P=edit_category&amp;group_id=' . $category_group . $session_param;
					if ($TMPL->tagdata != '')
					{ 
						return str_replace('{publish_that_url}', $url, $TMPL->tagdata);  
					}
					else 
					{	
						$alt = "Publish a new category";
						return $this->_get_link($url, $alt, $TMPL->fetch_param('icon'));
					}
				}
			}
			return;
		}
	}	

	// image: return a url or a link for an publish_that:image tag
	function image() 
	{
		global $TMPL, $PREFS, $DB, $SESS;
		$this->_debug("handling an image");
		$can_publish = FALSE;

		if (is_numeric($gallery_id = ($TMPL->fetch_param('gallery_id'))))
		{
			$this->_debug("parameters OK");

			// verify that they either have an admin session, or don't need one
			if ($this->_session_ok() === TRUE)
			{

				if ($this->_authz_superadmin() === TRUE)
				{
					$can_publish = TRUE;
				}
				else 
				{
					// they must be authorized to admin the gallery module.  cached to eliminate redundant queries.
					if (! isset($SESS->cache['publish_that']['gallery']))
					{
						$query = $DB->query("SELECT * FROM exp_module_member_groups mg, exp_modules mm WHERE mg.module_id = mm.module_id AND mm.module_name = 'Gallery' AND mg.group_id = " . $SESS->userdata['group_id']);

						// unlike weblogs, template groups, there are no more granular permissions to cache.  they either can or cannot admin gallery module.
						$SESS->cache['publish_that']['gallery'] = $query->num_rows > 0 ? TRUE : FALSE;
						$this->_debug("caching result of gallery authz");
					}

					if ($SESS->cache['publish_that']['gallery'] === TRUE)
					{
						// they must have these perms to edit 
						$can_publish = ($SESS->userdata['can_access_cp'] == 'y') && ($SESS->userdata['can_access_modules'] == 'y');	
						$this->_debug("control panel permission check: $can_publish");
					}
				}
			
				if ($can_publish)
				{
					// if admin sessions are required, get the session id
					$session_param = ($this->session_required == TRUE) ? '&amp;S=' . $SESS->cache['publish_that']['session_id'] : '';

					$url = $PREFS->core_ini['cp_url'] . '?C=modules&amp;M=gallery&amp;P=entry_form&amp;gallery_id=' . $gallery_id . $session_param;
					if ($TMPL->tagdata != '')
					{ 
						return str_replace('{publish_that_url}', $url, $TMPL->tagdata);  
					}
					else 
					{	
						$alt = "Publish a new image";
						return $this->_get_link($url, $alt, $TMPL->fetch_param('icon'));
					}
				}
			}
			return;
		}
	}
	
  // ----------------------------------------
  //  Plugin Usage
  // ----------------------------------------

  // This function describes how the plugin is used.
  //  Make sure and use output buffering

  function usage()
  {
  ob_start(); 
  ?>

This plugin will put small pencil icons in the live site, but only for users who are logged in, have an administrative session, and have permission to add that particular resource.

All parameters are required, except "icon".

The simplest usage is simply this tag without any enclosed text:

Entries:
{exp:publish_that:entry weblog_id="{weblog_id}"} 

Templates:
{exp:publish_that:template template_group="2"} 

Categories:
{exp:publish_that:category category_group="{category_group}"}

Gallery Images:
{exp:publish_that:image gallery_id="{gallery_id}"}

If you wish to use a different icon image, you can put your own image there, as long as the filename ends in “_icon.gif”.  Call alternate icons like so:
{exp:publish_that:template id="42" icon="custom"}

Advanced Use:

You can use the same tag, but enclose some text within it.  The plugin will do the permission check, but leave the presentation of the link to you. This allows you to change the display to whatever you'd like, and you could have entire instructions or alternate content displayed by this plugin.

You can even point to a front-end editing template (stand alone editing form) using this option.

{exp:publish_that:entry weblog_id="{weblog_id}"} 
<a href="{publish_that_url}">Create a new entry in this weblog/section/channel</a>
{/exp:publish_that:entry}

{exp:publish_that:template template_group="2"} 
<a href="{publish_that_url}">Create a new template in this template group</a>
{/exp:publish_that:template}

{exp:publish_that:category category_group="{category_group}"}
<a href="{publish_that_url}">Create a new category in this category group</a>
{/exp:publish_that:category}

{exp:publish_that:image gallery_id="{gallery_id}"}
<a href="{publish_that_url}">Create a new gallery image in this gallery</a>
{/exp:publish_that:image}

Image editing only applies to images in the gallery module.

	<?php
  $buffer = ob_get_contents();
	
  ob_end_clean(); 

  return $buffer;
  }
  // END

}

?>

